codeseg   segment para public 'CODE'
          assume   cs:codeseg,ds:codeseg,es:codeseg
extrn _parc:far,_pards:far,_parni:far,_parnl:far,_parnd:far
extrn _parl:far
extrn _retc:far,_retds:far,_retni:far,_retnl:far
extrn _retnd:far,_retl:far
extrn _parinfo:far
;
;
;  LOCAL DATA FOR USE BY THESE ROUTINES ONLY
;
semaphore db  0
;
; interrupt vectors stored here
;
installed  db  0
interrupt8  dd  0
intip8     equ word ptr interrupt8
intcs8     equ word ptr interrupt8 + 2
interrupt16 dd  0
intip16    equ word ptr interrupt16
intcs16    equ word ptr interrupt16 + 2
;
; timer
trow         db  0    ; row location of time
tcolumn      db  0    ; column for time
tcolor       db  7    ; time and date attribute
;
; alarm
arow         db  0    ; row location of time
acolumn      db  0    ; column for time
acolor       db  7    ; time and date attribute
;
;
cursor       dw  0    ; save cursor (dx reg)
pager        dw  0    ; save page   (bx reg)
;
months       db  "January   "
             db  "February  "
             db  "March     "
             db  "April     "
             db  "May       "
             db  "June      "
             db  "July      "
             db  "August    "
             db  "September "
             db  "October   "
             db  "November  "
             db  "December  "
days       db  "Sun"
           db  "Mon"
           db  "Tue"
           db  "Wed"
           db  "Thu"
           db  "Fri"
           db  "Sat"
;
timestring equ this byte
hours   db "00:"
minutes db "00:"
seconds db "00"
;
;
tone_sw     db  1    ; turns tone off and on (starts as on)
clock_cx    dw  0
clock_dx    dw  0
clock_count db 0     ; check only every eighteen ticks
string_len  dw 0      ; length of string to go to int 16
string_index dw  0
go16sw      db  0

alarm_init     proc     far
public alarm_init
           PUSH     BP                  ;Save registers
           MOV      BP,SP
           PUSH     DS
           PUSH     ES
           mov      ax,cs
           mov      ds,ax
           mov      es,ax
           mov      semaphore,1 ; turn off semaphore
;
;1.  varL = ALARM_INIT(expN)
;
;           where:       expN = 1 when the new interrupts are
;                               to be installed, and 0 when
;                               the new interrupts are to be un-
;                               installed
;
;           returns:     varL = .T. for installed or .F. removed
;
;
; step 0. check to see what we got passed
       call check_parms
       cmp  ax,1
       je   short init_0
init_error:
       stc           ; I don't know how to return an error
   jmp leave_install ; but returning nothing should do it.
exit_install:
          call   ret_log
leave_install:
;   Registers must be restored
          mov       semaphore,0
          POP       ES
          POP       DS
          POP       BP
          RET
init_0:
       call  check_nth
       cmp   ax,2
       jne   init_error
; step 1. get the expression in order to find out what to do
;
       mov   ax,1    ; parm 1
       call  get_int ; get it in ax
;
       cmp   ax,1
       je    install_ints
       cmp   ax,0
       jne   init_error
de_install_ints:
; de_install interrupts
       cmp    installed,0
       jne    do_deinstall
       jmp   init_error
do_deinstall:
        mov   ax,1
        jmp   exit_install
; int 8 first
        push  ds
        lds   bx,interrupt8
        mov   ax,2508h
        int   21h
        pop   ds
; now the keyboard
        push  ds
        lds   bx,interrupt16
        mov   ax,2516h
        int   21h
        pop   ds
        mov   ax,0
        jmp   exit_install
install_ints:
; see if they are already installed
        cmp   installed,1
        jne   do_install
        mov   ax,1
        jmp   exit_install
do_install:
; install interrupts
;
; set the interrupt 8 to us
        mov   ax,3508h
        int   21h
        mov   intcs8,es
        mov   intip8,bx
        mov   ax,2508h
        lea   dx,clock
        int   21h
; set the interrupt 16 to us
        mov   ax,3516h       ; get the current 16
        int   21h
        mov   intcs16,es     ; save it
        mov   intip16,bx
        mov   ax,2516h
        lea   dx,int16       ; procedure for new 16
        int   21h
;
        mov   ax,1
        jmp   exit_install
alarm_init     endp
;
;
get_parms proc near
;Clipper type |  C type     | Function     | CPU registers
;------------------------------------------------------------------
;char         | char *      | _PARC        | AX:BX (segment:offset)
;date         | char *      | _PARDS       | AX:BX (segment:offset)
;numeric      | int         | _PARNI       | AX
;numeric      | long        | _PARNL       | AX:BX
;numeric      | double      | _PARND       | AX:BX:CX:DX
;logical      | int         | _PARL        | AX
;char         | char *      | _RETC        | (segment:offset)
;date         | char *      | _RETDS       | (segment:offset)
;numeric      | int         | _RETNI       | 1 register
;numeric      | long        | _RETNL       | 2 registers
;numeric      | double      | _RETND       | 4 registers
;logical      | int         | _RETL        | 1 register
;------------------------------------------------------------------
;
;When using the _PAR functins, follow the pseudo sequence below:
;
;        1 - Move the parameter number to be passed into AX
;        2 - Push AX
;        3 - Call the appropriate "_PAR..." for that type
;        4 - The parameter is received in registers, either by
;             value or by pointer.
;        5 - Restore the stack
;
; (these have the parameter number in the AX register on entry
;
get_char:
           PUSH     AX                  ;putting n on the stack
           CALL     _PARc               ;Put seg:off in AX:BX
           ADD      SP,2                ;Fix up stack
           ret
get_int:
           PUSH     AX                  ;putting n on the stack
           CALL     _PARni              ;Put int in AX
           ADD      SP,2                ;Fix up stack
           ret
ret_log:
; 1 = true, 0 = false
           PUSH     AX                  ;putting tf on the stack
           CALL     _retl               ;Put int in AX
           ADD      SP,2                ;Fix up stack
           ret
get_parms endp
;
;
;
check_parms  proc near
; check to see that how many functions were passed to us
          XOR       AX,AX               ;Zero AX
          PUSH      AX                  ;Place on stack
          CALL      _PARINFO            ;Get count in AX
          ADD       SP,2                ;Fix up stack
          ret
;The type of each parameter is available by placing the parameter
;number on the stack and again CALLing _PARINFO.  The type is
;returned in the AX register.  The following equates show the
;value returned for each type.
;
;UNDEF     EQU   0
;CHARACTER EQU   1
;NUMERIC   EQU   2
;LOGICAL   EQU   4
;DATE      EQU   8
;ALIAS     EQU   16
;
;To get the type of the Nth parameter:
;
check_nth:
; this has the number of the parm to check in ax
;         MOV       AX,N                ;Place parameter number
          PUSH      AX                  ;on the stack
          CALL      _PARINFO            ;Get type in AX
          ADD       SP,2                ;Fix up stack
          ret
check_parms  endp
;
alarm_tone     proc     far
public alarm_tone
;2.  varL = ALARM_TONE(expN)
;
;           where:       expN = 1 to toggle the bell on and 0
;                               to toggle it off
;
;           returns:     varL = .T. if bell will sound and .F. if
;                               bell is muted
;
; this routine controls whether or not a tone is heard
; the tone is the blip blop sequence. it is not
; neccessary to make any calls here just set the tone switch
           PUSH     BP                  ;Save registers
           MOV      BP,SP
           PUSH     DS
           PUSH     ES
           push     ax
           mov      ax,cs
           mov      ds,ax
           mov      es,ax
           mov      semaphore,1 ; turn off semaphore
           pop      ax
           mov      tone_sw,al
           push     ax
          call   ret_log
           pop      ax
          mov       semaphore,0
          POP       ES
          POP       DS
          POP       BP
          RET
alarm_tone     endp
;
;
alarm_disp     proc     far
public alarm_disp
;3.  varL = ALARM_DISP(expN1,<expN2>,<expN3>,<expN4>,<expN5>)
;
;           where:      expN1 = 1 to toggle the display of date,
;                               three-letter day, and time (12
;                               hour format, and with "AM" or "PM"
;                               suffixed) on screen, and 2 to
;                               toggle display off
;
;                       expN2 = 1 to include display of seconds
;                               and 0 to omit display of seconds
;
;                       expN3 = row number for display (not
;                               required if expN1 is false)
;
;                       expN4 = column number for display (not
;                               required if expN1 is false)
;
;                       expN5 = color byte value for display (not
;                               required if expN1 is false)
;
;           returns:     varL = .T. if alarm display is on, and
;                               .F. if alarm display is off
;
alarm_disp     endp
;
;
;
alarm_key     proc     far
public alarm_key
;4.  varN = ALARM_KEY(expN)
;
;           where:       expN = value which would be use in a
;                               "SET KEY <expN> to <PROG>" state-
;                               ment
;
;           returns:     varN = current "SET KEY" <value>
;
alarm_key      endp
;
;
;
alarm_set      proc     far
public alarm_set
;5.  varC = ALARM_SET(expN,<expC> or <expN>)
;
;           where:       expL = 1 to set a new alarm time, or 0
;                               to cancel the current alarm time
;
;                        expC = character string (HH:MM:SS)
;                               containing the time for the alarm
;                               expressed in 24 hour format
;
;           returns:     varC = character string (HH:MM:SS)
;                               containing the time for which the
;                               alarm is currently set expressed
;                               in a 24 hour format; should return
;                               a null string if no time is set
;
alarm_set      endp
;
;
timer          proc     far
public timer
;6.  varL = TIMER(expN)
;
;           where:       expN = 1 to start the timer, 2 to
;                               stop the timer, and 0 to reset
;                               the timer
;
;           returns:     varL = 1 if timer is running, 2 if
;                               timer is stopped but not
;                               reset, and 0 if timer is not
;                               running and has been reset
;
timer          endp
;
;
timer_disp     proc     far
public timer_disp
;7.  varL = TIMER_DISP(expN1,<expN2>,<expN3>,<expN4>)
;
;           where:      expN1 = 1 to toggle the display of
;                               timer (24 hour format)
;                               including seconds on screen
;                               and 0 to toggle display off
;
;                       expN2 = row number for display (not
;                               required if expN1 is false)
;
;                       expN3 = column number for display (not
;                               required if expN1 is false)
;
;                       expN4 = color byte value for display (not
;                               required if expN1 is false)
;
;           returns:     varL = .T. if timer display is on and
;                               .F. if timer display is off
;
timer_disp      endp
;
;
write_time  proc  far
public write_time
; this prints out the time in 24 hour format at dx
;
        push  dx
        mov   ax,cs
        mov   ds,ax
        mov   es,ax
        mov   ah,3
        mov   bx,0
        int   10h
        mov   cursor,dx
; get the current time using bios call
        mov   ah,00h
        int   1ah
; the registers are set as follows:
   ;cx is high order clock count
   ;dx is low order clock count
; measured in ticks since midnight
; must turn it into seconds
;
; there are 1193180/65536 counts per second
; this looks like the hours are in cx
; how come I never noticed that before?
;
    push  dx
    mov   al,cl
    mov   bx,cx
    mov   cl,10
    mov   ah,0
    div   cl
    or    ax,3030h
     mov   hours,al
     mov   hours+1,ah
     mov   ax,bx
     add   ax,2
     mov   cx,3
     mov   dx,0
     div   cx
     mov   bx,ax
     pop   ax
; now have to convert to minutes
; there are 8739.11 ticks in 8 minutes, closest fraction
     mov   cx,1
     mov   dx,0
     clc
     rcl   ax,1
     rcl   dx,1
     clc
     rcl   ax,1
     rcl   dx,1
     clc
     rcl   ax,1
     rcl   dx,1
; multiplied ax, by 8 (beleive it or not
     mov   cx,8739 ; ticks per minute
     div   cx
; dx has the ticks,  left over seconds
     push  dx
     mov   cl,10
     div cl
    or    ax,3030h
     mov   minutes,al
     mov   minutes+1,ah
; there are 18.206 ticks per second
; and 1165.21 in 64 seconds
; but its already been x'd by 8
     pop   ax
     mov   dx,0
     mov   cx,8
     mul   cx
     mov   cx,1165
     div   cx
     mov   cl,10
     sub   ax,bx  ; seconds correction from assump about hours
     div cl
    or    ax,3030h
     mov   seconds,al
     mov   seconds+1,ah
; now write it
        lea   si,timestring
        mov   dl,0
        mov   dh,3
        mov   cx,8
        call  print_string
exit_write_time:
; reset the current cursor
        mov   ah,2
        mov   dx,cursor
        mov   bx,0
        int   10h
        pop   dx
        ret
write_time  endp
;
;
;
write_date  proc  far
public write_date
; this routine writes the date at the locations specified
; in column and row with the attribute color
;
; get cursor position to reset later
;
        mov   ax,cs
        mov   ds,ax
        mov   es,ax
        mov   ah,3
        mov   bx,0
        int   10h
        mov   cursor,dx
;
; get the current date
; use DOS call
        mov   ah,2ah
        int   21h
; cx has the year, dh has month, dl has date, al has dow
        push  cx
        push  dx
        mov   ah,0
        mov   cl,3
        mul   cl
        mov   bx,ax
        lea   si,days[bx]
        mov   dl,tcolumn
        mov   dh,trow
        mov   cx,3
        call  print_string
        mov   al,' '
        call  print_byte
; pop out the month and push it
        pop   ax
        push  ax
; print the month
        mov   al,ah
        mov   ah,0
        mov   cl,10
        mul   cl
        mov   bx,ax
        lea   si,months[bx]
        mov   cx,10
        call  print_string
        mov   al,' '
        call  print_byte
; now the day of the month (two digits)
        pop   ax
        mov   ah,0
        mov   cl,10
        div   cl
        or    ax,3030h
        push  ax
        call  print_byte
        pop   ax
        mov   al,ah
        call  print_byte
        mov   al,','
        call  print_byte
        mov   al,' '
        call  print_byte
        mov   al,'1'
        call  print_byte
        mov   al,'9'
        call  print_byte
; print the year
        pop   ax
        sub   ax,1900
        mov   ah,0
        mov   cl,10
        div   cl
        or    ax,3030h
        push  ax
        call  print_byte
        pop   ax
        mov   al,ah
        call  print_byte
;
; reset the current cursor
        mov   ah,2
        mov   dx,cursor
        mov   bx,0
        int   10h
        ret
write_date  endp
print_string proc near
ps_loop:
        push  cx
        cld
        lodsb
        cmp   al,' '
        jne   print_it
        pop   cx
        ret
print_it:
        call  print_byte
        pop    cx
        loop  ps_loop
        ret
print_string  endp
print_byte  proc  near
; prints al at position dx
        push  ax
        mov   ah,2
        mov   bx,0
        mov   ah,2
        mov   bx,0
        int   10h
        pop   ax
        mov   cx,1
        mov   bh,0
        mov   bl,tcolor
        mov   ah,9
        int   10h
        inc   dl
        ret
print_byte  endp
;
clock  proc near
       iret
clock  endp
int16  proc near
       iret
int16  endp
;
; the tone routines produce glisandos based on the value of si
blip    proc  near
        push  si
        mov   si,30
        call  tone
        pop   si
        ret
blip    endp
blop    proc  near
        push  si
        mov   si,-30
        call  tone
        pop   si
        ret
blop    endp
tone    proc  near
        cmp   tone_sw,1
        je    do_tone
        ret
do_tone:
        push  ax
        push  bx
        push  cx
        push  dx
        mov   dx,3    ; tone rate
        mov   bx,240  ; starting freq
        mov   cx,750  ; ending frequency
        cmp   si,0
        jg    do_blip
; here we do blop
        mov   bx,630  ; starting freq
        mov   cx,150  ; ending frequency
do_blip:
; turn on tone
i0:
        push  cx
        push  bx
        in     al,61h
        or     al,3
        out    61h,al
; loop parameters
        mov    cx,bx
 ; set frequency
        mov    dx,12h
        mov    ax,34deh
        div    cx
        mov    cx,ax
; set tone
        mov    al,cl
        out    42h,al
        mov    al,ch
        out    42h,al
i2:     loop i2
        pop    bx
        pop    cx
        cmp    bx,cx
        je     ip3
        add    bx,si
        jmp    i0
ip3:
 ;turn off tone
        in  al,61h
        and al,11111100b
        out 61h,al
        pop  dx
        pop  cx
        pop  bx
        pop  ax
        ret
tone    endp
;
codeseg   ends
          end

    The timer functions do not need an alarm, and to save 
programming complexity, the timer should have its seconds ticked 
in alignment with the seconds in the alarm functions.  By this I 
mean that when the program calls the timer function, if the new 
interrupt shows the system clock to be between boundaries of the 
exact changing of seconds, then the timer can be permitted to log 
the first second of the timer count whenever the next second is 
recorded in the system clock, even if less than a second has 
elapsed.  In the applications I have in mind, timing to the 
precise fraction of a second is not that critical.  If you can do 
actual timer counts independent of the system clock such that the 
seconds of the timer and the seconds of the time shown by the 
alarm do not change simultaneously (due to the timer being 
started between seconds) without slowing down the system and 
without lots of extra programming time, feel free to do so 
because I can foresee an application where such functions may be 
useful.  Again this last feature is not mandatory and can be 
omitted in the name of saving money.

    Let me know what you think this will cost if it appears it 
will go above $400-500.  Many thanks!

                                  Bob DeFazio
                                  (213) 552-1681
                                  call collect
print macro string
     lea  dx,string
     mov  ah,9
     int  21h
     endm
get  macro string
     lea  dx,string
     mov  ah,0ah
     int  21h
     endm
cSEG        SEGMENT PARA PUBLIC 'CODE'
        assume cs:cseg,ds:cseg,es:cseg
        ORG        100H                        ;SKIP TO END OF THE PSP
ENTER:        JMP        setres                       ;COM FILE ENTRY ALWAYS AT 100H
START        PROC        NEAR
int16:
; here we fake a keyboard
 ; let keyboard do its thing
        sti
        cmp  cs:go16sw,0
        jne  got_kb
jmp_back:
        jmp  cs:[interrupt16]
got_kb:
        cmp  ah,1
        jg   jmp_back
        push bx
        mov  bx,cs:string_index
        cmp  ah,1
        je   check_kbd_only
        add  cs:string_index,2
        sub  cs:string_len,1
check_kbd_only:
        mov  ax,word ptr cs:string_area[bx]
        cmp  cs:string_len,0
        jg   return_string
        mov  cs:go16sw,0
return_string:
; set zero flag 0
        mov  bx,0
        cmp  bx,1
        pop  bx
        clc
        db   0cah,02,00  ; ret far 2
;
; here is the clock interrupt
;
clock:
; we are entering here due the clock being ticked
        pushf
 ; let clock do its thing
        call cs:[interrupt8]
        sti
        inc  cs:clock_count
        cmp  cs:clock_count,12
        jg   got_clock
        iret
got_clock:
        cmp  cs:go16sw,1
        jne  go_on_clock
        iret
go_on_clock:
        mov  cs:clock_count,0
        push  ax
        push  bx
        push  cx
        push  dx
        mov   ah,00h
        int   1ah
; the registers are set as follows:
   ;cx is high order clock count
   ;dx is low order clock count
        and  dx,0fff0h
        cmp  cs:clock_cx,cx
        jne  go_back
        cmp  cs:clock_dx,dx
        jne  go_back
        jmp  gotcha
go_back:
;return to wherever
        pop   dx
        pop   cx
        pop   bx
        pop   ax
        iret
gotcha:
        mov  cs:go16sw,1
        mov  ah,14
        mov  al,7
        int  10h
        jmp  go_back
string_area dw 203  dup(0)
START        ENDP
setres  proc  near
        print   version
check_vers:
        mov  ah,30h
        int  21h
        cmp  al,2
        jge  check_command
        print   mess3
        int  20h
check_command:
; here we check for command line parameters
        cmp byte ptr ds:[80h],7
        jle  gettime1
        mov  time+1,0
        mov  cl,byte ptr ds:[80h]
        mov  ch,0
        mov  si,81h
        lea  di,time+2
com_space_scan:
        cld
        lodsb
        cmp al,13
        je  gettime1
        cmp al,' '
        je  com_space_scan_end
        jmp com_move_time
com_space_scan_end:
        loop com_space_scan
        jmp gettime1
com_move_time:
 ; got a non space byte
       cmp  al,'0'
       jl  gettime1
       cmp  al,'9'
       jg  gettime1
       stosb
       inc time+1
       lodsb
       cmp  al,' '
       je  com_move_command
       loop com_move_time
       jmp  gettime1
com_move_command:
       lea di,string_area
com_m_com_loop:
       lodsb
       mov ah,0
       stosw
       inc  string_len
       loop com_m_com_loop
; we have done a command line instead of an input
       jmp   got_time_com
gettime:
        print mess2
gettime1:
; get the time of the wakeup
        print mess1
        get time
got_time_com:
        cmp  time+1,6
        jne  gettime
        mov  cx,6
        lea  si,time+2
        cld
ctime_loop:
        lodsb
        cmp   al,'0'
        jl    gettime
        cmp   al,'9'
        jg    gettime
        loop  ctime_loop
; convert each word to binary
        mov   cx,3
        lea   si,time+2
        mov   di,si
        cld
ctime_loop2:
        push  cx
        lodsw
        and   ax,0f0fh
        mov   bl,ah
        mov   ah,0
        mov   cl,10
        mul   cl
        add   al,bl
        stosw
        pop   cx
        loop  ctime_loop2
; check each word for valid range
        mov   ax,word ptr time+6
        cmp   al,60
        jl    good_second
        jmp   gettime
good_second:
        mov   bx,ax
        mov   ax,word ptr time+4
        cmp   al,60
        jl    good_second2
        jmp   gettime
good_second2:
        mov   cl,60
        mul   cl
        add   bx,ax
        mov   ax,word ptr time+2
        cmp   al,24
        jl    good_second3
        jmp   gettime
good_second3:
        mov   cx,3600
        mov   dx,0
        mul   cx
        add   ax,bx
        jnc   seconds_nc
        inc   dx
seconds_nc:
; the number in dx:ax is the total number of seconds
; now must mul by 18.21 to get
; mov the dx to bx to save it for future use
        mov    bx,dx
        mov    si,ax
; si saves the low bytes of seconds
; clock ticks
        mov   clock_cx,0   ; cx is high
        mov   clock_dx,0   ; dx is low
; lets work on bx first
        mov   ax,bx
        mov   dx,0
        mov   cx,18
        mul   cx
        mov   clock_cx,ax
; thats the 18 part, now the .21 part must be dealt with
        mov   ax,bx
        mov   dx,0
        mov   cx,3383
        mul   cx
        mov   cx,16384
        div   cx
        add   clock_cx,ax
; dx has the remainder, the fractional part of the division
        mov   ax,dx
        mov   dx,0
        mov   cx,4
        mul   cx
        mov   clock_dx,ax
; now seconds hi are done
; work on seconds low
        mov   ax,si
        mov   dx,0
        mov   cx,18
        mul   cx
        add   clock_cx,dx
        add   clock_dx,ax
        jnc   clock_nc1
        inc   clock_cx
clock_nc1:
; now the fraction .206 which is about 3383/16384
        mov   ax,si
        mov   dx,0
        mov   cx,3383
        mul   cx
        mov   cx,16384
        div   cx
        add   clock_dx,ax
        jnc   clock_nc2
        inc   clock_cx
clock_nc2:
        and  clock_dx,0fff0h
; verify the clock is right
ask_ok:
        cmp   string_len,0
        jne   got_all_time
        print mess4
        get   yn_in
        cmp   yn,'Y'
        je    got_all_time
        cmp   yn,'y'
        je    got_all_time
        cmp   yn,'N'
        je    get_more_time
        cmp   yn,'n'
        je    get_more_time
        print crlf
        jmp   ask_ok
get_more_time:
        jmp   gettime
got_all_time:
; all done clock stuff
; get the string
        cmp   string_len,0
        je    no_string_yet
        jmp   got_all_string
no_string_yet:
        print mess5
        mov   pluscount,0
        mov   string_len,0
        lea   di,string_area
        mov   cx,200
char_loop:
        push  cx
        mov   ah,0
        int   16h
        cmp   ax,4e2bh
        jne   no_plus
        inc   pluscount
        cmp   pluscount,3
        je    got_plus
        jmp   by_plus
no_plus:
        mov   pluscount,0
by_plus:
        inc   string_len
        stosw
        mov   ah,14
        cmp   al,13
        jne   no13
        int   10h
        mov   al,10
        mov   ah,14
no13:
       int    10h
        loop  char_loop
; fell out of loop
        add   string_len,2
got_plus:
        sub   string_len,2
; finished entering string
;
; check it
ask_ok2:
        print mess6
        get yn_in
        cmp   yn,'Y'
        je    got_all_string
        cmp   yn,'y'
        je    got_all_string
        cmp   yn,'N'
        je    get_more_string
        cmp   yn,'n'
        je    get_more_string
        cmp   yn,'A'
        je    abort
        cmp   yn,'a'
        je    abort
        print crlf
        jmp   ask_ok2
abort:
        print crlf
        int  20h
get_more_string:
        jmp  no_string_yet
got_all_string:
 ;      mov the whole thing to 60h
        lea cx,setres
        sub cx,100h
        mov si,100h
        mov di,60h
        cld
rep     movsb
; set ds and es registers different
        push  cs
        pop   ax
        sub   ax,0ah
        mov   ds,ax
        mov   es,ax
        int   20h
version db  'WAKEUP.COM Resident Wakeup call',13,10
        db  'Written by Keith P. Graham 9/19/86',13,10
        db  'For the users of PC Rockland BBS (914) 353-2176'
crlf   db   13,10,'$'
mess1   db  13,10,'Enter time for wakeup as HHMMSS (military time) :$'
mess2   db  13,10,'Time must be six numeric digits, try again',13,10,'$'
mess3   db  'Wrong DOS version, Must be 2.0 or later',13,10
mess4   db  13,10,'Is the time correct? (y/n): $'
mess5   db 13,10,'Enter Keystrokes to be entered ar the specified time.',13,10
        db  'You have up to 200 characters you can enter.',13,10
        db  'When you are done, press the + on the numeric keypad',13,10
        db  'three times.',13,10,10,'   ENTER:',13,10,'$'
mess6   db  13,10,'Is the sequence correct (Y)es, (N)o, (A)bort :$'
pluscount db 0
yn_in   db  2,0
yn      db  '  '
time    db  7,0,'        '
setres  endp
CSEG        ENDS
        END        enter

ASSEMBLY back to CLIPPER
------------------------
There is also a set of functions that begins with "_RET" for
returning values to your Clipper programs.  Here is a chart,
corresponding to the one above.  Notice that the address of the
return string is placed on the stack for char types and that the
value to be returned is placed on the stack for numerics.  Since
the DO <procedure> WITH... does not return any values, it can
only use the _RET function.
The BP, DS, and ES registers must always be saved upon entry to
your assembly language routine.  Before calling these functions,
always restore BP, DS, and, ES.  After calling the function,
always fix up the stack.  Failure to restore the environment
this way will usually hang the machine.

The following is a shell of a routine to get and return a
character value.  Note that all the functions CALLed must be
declared as external or the assembler will return an error message.

          PUBLIC    <function>

;Declare the Clipper calls as external and far
         EXTRN    _PARC:FAR
         EXTRN    _RETC:FAR

CODESEG   SEGMENT   BYTE 'CODE'
          ASSUME    CS:CODESEG

;*** Your code goes here ***

;Return pointer to Clipper
<function> ENDP
CODESEG ENDS
          END       <function>


Here is a function that takes a string as a parameter and
returns the string in reverse order.

;Function...:TURN.ASM
;Syntax.....:? TURN(<expC>)
;Note.......:reverses the order of the string passed
;            can only process a MAX of 60 characters
;
public  TURN            ; this public statement makes this
;                       ; routine accessible to the 'public'
;
extrn   _PARC:far       ; Clipper's character 'getter'
extrn   _RETC:far       ; Clipper's character 'returner'
;
;
DGROUP  GROUP   datasg  ; Clipper's Data Segment

; the 'public' in the next statement combines the datasg
; to Clipper's DGROUP group
datasg  segment public  'DATA'

RETVAL  db      60 dup(" ")     ; init with 60 spaces
VAL1    dw      0               ; init with 0
VAL2    dw      0               ; init with 0

datasg  ends                    ; end of datasg (in DGROUP)
;
;
_prog   segment
        assume  cs:_prog,ds:DGROUP,es:DGROUP

TURN    proc    far             ; far process
        push    bp              ; preserve return address
        mov     bp,sp           ; move stack pointer
                
        mov     ax,1            ; get first para
        push    ax              ; push AX
        call    _PARC           ; call Chara "getter"
        add     sp,2            ; restore stack
        mov     VAL1,bx         ; get OFFSET address 
        mov     VAL2,ax         ; get SEGMENT address 

        push    ds              ; preserve DS
        push    es              ; preserve ES

        cld                     ; clear direction flag (L-R)
        mov     di,offset DGROUP:RETVAL ; get RETVAL's offset
                                        ; from DGROUP
        mov     ax,DGROUP       ; get DGROUP segment
        mov     es,ax           ; and place in ES

; in the next three statements, the order that it appears is
; important because the values of SI & AX must first be taken 
; from the DATASG extension before assigning the DGROUP segment
; address to DS

        mov     si,VAL1
        mov     ax,VAL2
        mov     ds,ax
                                
        
        xor     ax,ax           ; zero out AX
        mov     al,[si]         ; move first SI value to AL
        cmp     al,0            ; if NUL (zero)
        jz      C3              ; jump to C3

        mov     cx,3ch          ; set CX for max 60 loops
        xor     bx,bx           ; zero-out BX
        xor     ax,ax           ; zero-out AX

A1:     mov     al,[si]         ; move SI value to AL
        cmp     al,0            ; compare with zero
        jz      A2              ; if zero, jump to A2:
        inc     si              ; increment SI
        inc     bx              ; increment BX
        loop    A1              ; loop to A1:

A2:     dec     si              ; decrement SI
        mov     cx,bx           ; move BX to CX
        xor     ax,ax           ; zero-out AX

A3:     mov     al,[si]         ; move  SI val to AL
        mov     es:[di],al      ; move  AL to DI
        dec     si              ; decrement SI
        inc     di              ; increment DI
        loop    A3              ; loop
        
C3:     xor     ax,ax           ; zero-out AX
        mov     es:[di],al      ; add null terminator to end
                                ; of RETVAL

        pop     es              ; restore
        pop     ds              ; restore

        pop     bp              ; restore

        mov     ax,DGROUP       ; get DGROUP address
        mov     ds,ax           ; assign to DS
        push    ds              ; push segment address
        mov     ax,offset DGROUP:RETVAL ; get RETVAL's offset
        push    ax              ; push offset address
        call    _RETC           ; Clipper's  "returner"
        pop     ax              ; restore
        pop     ds              ; restore

        ret                     ; actual ret to caller
TURN    endp                    ; end of process

_prog   ends                    ; end of segment
        end                     ; end of program



; Filename: EXTENDA.ASM
; Program.: Clipper Extended Library
; Authors.: Tom Rettig, Brian Russell
; Date....: November 1, 1985
; Notice..: Placed in the public domain by Tom Rettig Associates.
;           Clipper is a trademark of Nantucket.
;           dBASE and dBASE III are trademarks of Ashton-Tate.
; Notes...: User-defined dBASE functions in 8086 assembly for Clipper.
;
;   Functions not in dBASE III or Clipper:
;      ISPRINTER() ::= True if parallel printer is online and ready.
;

   NAME   EXTENDA

   PUBLIC ISPRINTER


; Clipper return value calls
   EXTRN   _RETC:FAR    ; return character string
   EXTRN   _RETDS:FAR   ; return date type from date string "YYYYMMDD"
   EXTRN   _RETL:FAR    ; return logical true or false
   EXTRN   _RETNI:FAR   ; return word as numeric
   EXTRN   _RETNL:FAR   ; return double word as numeric
   EXTRN   _RETND:FAR   ; return floating point as numeric

_PROG SEGMENT

         ASSUME CS:_PROG

;------------------------------------------------------------------------------
;  ISPRINTER()
;  Syntax: ISPRINTER()
;  Return: Logical true if the printer is online and ready, otherwise false

ISPRINTER PROC FAR

         MOV      AH,2H          ; printer status function
         MOV      DX,0H          ; which printer to check
         INT      17H            ; read printer status

         XOR      BX,BX          ; false
         CMP      AH,90H         ; not busy or selected (90h = 10010000)
         JNE      RET_ISPRINTER  ; return false if other than above
         MOV      BX,1           ; true

RET_ISPRINTER:
         PUSH     BX             ; put return value on the stack
         CALL     _RETL          ; return logical value to Clipper
         POP      BX             ; restore the stack

         RET

ISPRINTER ENDP

_PROG ENDS

         END

; EOF Extenda.asm -------------------------------------------------------------
