clear		equ	256		;Clear code
eof		equ	257		;End of file marker
first_free	equ	258		;First free code
maxmax		equ	4096		;Max code + 1


;Hash table entry
hash_rec	struc
first	dw	?			; First entry with this value
next	dw	?			; Next entry along chain
char	db	?			; Suffix char
hash_rec	ends


;Start writing code
cseg	segment para public 'CODE'
	assume	cs:cseg,ds:cseg,es:cseg
extrn hash_seg:word,targ_buff:word,targ_buff_size:word
extrn curr_buff:word,curr_buff_size:word
;
prefix_code	dw	?
free_code	dw	?
max_code	dw	?
nbits		dw	?
k		db	?


bit_offset	dw	?
siloc   	dw 	0

compress	proc	near
public compress
        mov     cs:bit_offset,0
        mov     siloc,0
        mov     di,0
        push    ds
        push    es
        mov     es,targ_buff
        mov     ds,cs:curr_buff
l1:	call	init_table		;Initialize the table and some vars
	mov	ax,clear		;Write a clear code
	call	write_code
	call	read_char		;Read first char
l4:	xor	ah,ah			;Turn char into code
l4a:	mov	cs:prefix_code,ax		;Set prefix code
	call	read_char		;Read next char
	jc	l17			;Carry means eof
	mov	cs:k,al			;Save char in k
	mov	bx,cs:prefix_code		;Get prefix code
	call	lookup_code		;See if this pair in table
	jnc	l4a			;nc means yes, new code in ax
	call	add_code		;Add pair to table
	push	bx			;Save new code
	mov	ax,cs:prefix_code		;Write old cs:prefix code
	call	write_code
	pop	bx
	mov	al,cs:k			;Get last char
	cmp	bx,cs:max_code		;Exceed code size?
	jl	l4			;less means no
	cmp	cs:nbits,12		;Currently less than 12 bits?
	jl	l14			;yes
	mov	ax,clear		;Write a clear code
	call	write_code
	call	init_table		;Reinit table
	mov	al,cs:k			;get last char
	jmp	l4			;Start over
l14:	inc	cs:nbits			;Increase number of bits
	shl	cs:max_code,1		;Double max code size
	jmp	l4			;Get next char
l17:
	mov	ax,cs:prefix_code	;Write last code
	call	write_code
	mov	ax,eof			;Write eof code
	call	write_code
        pop     es
        pop     ds
	mov	ax,cs:bit_offset	;Make sure buffer is flushed to file
	mov	cx,3			;convert bits to bytes
	xor	dx,dx
        add     ax,7
	shr	ax,cl
        mov     targ_buff_size,ax
        ret
compress	endp

init_table	proc	near
        push    di
	mov	cs:nbits,9			;Set code size to 9
	mov	cs:max_code,512		;Set max code to 512
	push	es			;Save seg reg
	mov	es,cs:hash_seg		;Address hash table
	mov	ax,-1			;Unused flag
	mov	cx,640			;Clear first 256 entries
	mov	di,0
rep	stosw				;Clear it out
	pop	es			;Restore seg reg
	mov	cs:free_code,first_free	;Set next code to use
        pop     di
	ret				;done
init_table	endp

write_code	proc	near
	push	ax			;Save code
	mov	ax,cs:bit_offset		;Get bit offset
	mov	cx,cs:nbits		;Adjust bit offset by code size
	add	cs:bit_offset,cx
	mov	cx,8			;Convert bit offset to byte offset
	xor	dx,dx
	div	cx
wc1:
	mov	di,ax			;Destination
	pop	ax			;Restore code
	mov	cx,dx			;offset within byte
	xor	dx,dx			;dx will catch bits rotated out
	jcxz	wc3			;If offset in byte is zero, skip shift
wc2:	shl	ax,1			;Rotate code
	rcl	dx,1
	loop	wc2
	or	al,byte ptr es:[di]	;Grab bits currently in buffer
wc3:	stosw				;Save data
	mov	al,dl			;Grab extra bits
	stosb				;and save
	ret
write_code	endp
read_char	proc	near
        push    si
        mov     si,cs:siloc
        cmp     si,cs:curr_buff_size
        jl      rd1
        pop     si
        stc
        ret
rd1:
	lodsb				;Read it in
        inc    cs:siloc
        pop   si
	clc				;Success
	ret
read_char	endp


lookup_code	proc	near
	push	ds			;Save seg reg
	mov	ds,cs:hash_seg		;point to hash table
	call	index			;convert code to address
	mov	di,0			;flag
	cmp	[si].first,-1		;Has this code been used?
	je	gc4			;equal means no
	inc	di			;set flag
	mov	bx,[si].first		;Get first entry
gc2:	call	index			;convert code to address
	cmp	[si].char,al		;is char the same?
	jne	gc3			;ne means no
	clc				;success
	mov	ax,bx			;put found code in ax
	pop	ds			;restore seg reg
	ret				;done
gc3:	cmp	[si].next,-1		;More left with this cs:prefix?
	je	gc4			;equal means no
	mov	bx,[si].next		;get next code
	jmp	gc2			;try again
gc4:	stc				;not found
	pop	ds			;restore seg reg
	ret				;done
lookup_code	endp

index		proc	near
	mov	si,bx			;si = bx * 5 (5 byte hash entries)
	shl	si,1			;si = bx * 2 * 2 + bx
	shl	si,1
	add	si,bx
	ret
index		endp

add_code	proc	near
	push	ds			;point to hash table
	mov	bx,cs:free_code		;Get code to use
	mov	ds,cs:hash_seg
	cmp	di,0			;First use of this cs:prefix?
	je	ac1			;equal means yes
	mov	[si].next,bx		;point last use to new entry
	jmp	short ac2
ac1:	mov	[si].first,bx		;Point first use to new entry
ac2:	cmp	bx,maxmax		;Have we reached code limit?
	je	ac3			;equal means yes, just return
	call	index			;get address of new entry
	mov	[si].first,-1		;initialize pointers
	mov	[si].next,-1
	mov	[si].char,al		;save suffix char
	inc	cs:free_code		;adjust next code
ac3:	pop	ds			;restore seg reg
	ret
add_code	endp
cseg	ends

	end
