; ====================================================================
; this area is for interrupts B,C,10 and 16
; It consists of 5 routines:
;
;   PROCEDURE INITCOM initializes the serial port vectors for
;   the serial port requested. DX must have the Port (0 or 1).
;   This does not change baud or bring up DTR.
;   Written
;
;   PROCEDURE INT16 sets the keyboard buffer back to normal
;
;   PROCEDURE INT10 sets the 10h function to go out the comport
;
;   PROCEDURE INT0B is a 1000 word circular buffer for com2
;
;   PROCEDURE INT0C same as int0b except for com1
;
;
;   PROCEDURE INT16 is a replacement for the BIOS routine 16h.
;   This routine looks for a keystroke from the buffer or the
;   keyboard. Local keyboard has precidence
;
;   PROCEDURE DECOM deinitializes the serial port vectors
;   Resets the keyboard vector if it is on.
;
;
;====================================================================
; This code should be an include file which is included in another  
; program's code. All procedures are near and assumes are for
; the cs and ds being the same segment.
;
;====================================================================
;    DATA AREAS
;====================================================================
com1loc  dw  0
com2loc  dw  0
com1on   db  0
com2on   db  0
set16    db  0
set10    db  0
set10a   db  0
int10old equ this dword
int10ip  dw  0
int10cs  dw  0
int16old equ this dword
int16ip  dw  0
int16cs  dw  0
int0cip  dw  0
int0ccs  dw  0
int0bip  dw  0
int0bcs  dw  0
istack   dw  128  dup(0)
isavess   dw 0
isavesp   dw 0
buffsize equ  8000
buffer  db   buffsize dup(0)
buffcount dw  0
buffpoint dw  0
bufftail  dw  0
bufffull  db  0
comline   db  0
commodem  db  0
semaphore1 db  0
semaphorec db  0
sem10   db 0 ; for int 10 bad bios guys
sem10a  db 0 ; for int 10 bad bios guys
sem16   db 0
comactive  db  0
comport    db  0  ; used by supporting routines to store comport
func16   db  0
mphase  dw  0
no_install  db  1
flow_control db 0
ecount  dw  0
scans  equ this byte
;    0   1   2   3   4   5   6   7   8   9   a   b   c   d   e   f
db 000,000,000,000,000,000,000,000,014,015,028,000,038,028,000,000   ; 0
db 000,000,000,031,000,000,000,000,000,000,000,001,000,045,000,000   ; 1
db 057,002,040,004,005,006,008,041,010,011,009,013,049,012,052,053   ; 2
db 011,002,003,004,005,006,007,008,009,010,039,039,035,013,052,053   ; 3
db 003,030,048,045,032,018,033,034,035,023,024,025,026,050,049,024   ; 4
db 025,016,019,031,020,022,047,017,045,021,044,026,043,027,007,012   ; 5
db 000,030,048,045,032,018,033,034,035,023,024,025,026,050,049,024   ; 6
db 025,016,019,031,020,022,047,017,045,021,044,025,043,027,041,083   ; 7
port21h  db 0
baud    db 10100000b   ; 2400
parity    db 0    ; none
databits  db  00000011b  ; 8,1
;
;====================================================================
;    PROCEDURES
;====================================================================
;
;====================================================================
;
;    DECOM Procedure uninitializes the com buffer
;
;====================================================================
setbaud proc near
    mov  al,parity
    or   al,baud
    or   al,databits
    mov  ah,0
    mov  dl,comport
    mov  dh,0
    int  14h
    ret
setbaud endp
;
decom proc near
    push  cs
    pop   ds
; have to turn off the interrupts
    cmp   cs:comactive,1
    je    de_1
    cmp   cs:comactive,2
    je    de_2
;   ret
de_1:
    call  unirq4
    jmp   check_interrupts
de_2:
    call  unirq3
check_interrupts:
; reset the interrupts
    cmp   set16,0
    je    no16_set
    mov   set16,0
    mov   dx,int16ip
    mov   ax,int16cs
    push  ds
    mov    ds,ax
    mov   ah,25h
    mov   al,16h
    int   21h
    pop    ds
no16_set:
    cmp   set10,0
    jne   unset10
    cmp   set10a,0
    je    no10_set
unset10:
    mov   set10,0
    mov   set10a,0
    mov   dx,int10ip
    mov   ax,int10cs
    push  ds
    mov    ds,ax
    mov   ah,25h
    mov   al,10h
    int   21h
    pop    ds
no10_set:
    ret
decom endp
;
;====================================================================
;
;    INIT10a Procedure
;
;====================================================================
;
init10a  proc   near
; this initializes 10h to the int10a routine for func 14 only
    cmp   set10,0
    jne   no_10seta
    cmp   set10a,0
    je    do_set10a
no_10seta:
    ret
do_set10a:
    mov   set10a,1
    push  es
    mov   ah,35h
    mov   al,10h
    int   21h    ; gets the interrupt
    mov   int10ip,bx
    mov   int10cs,es
    pop   es
; now have 16 point to int16 procedure
    mov   ah,25h
    mov   al,10h
    lea   dx,int10a
    int   21h
    ret
init10a endp
;
;
;====================================================================
;
;    INIT10 Procedure
;
;====================================================================
;
init10  proc   near
; this initializes 10h to the int10 routine
    cmp   set10a,0
    jne   no_10set
    cmp   set10,0
    je    do_set10
no_10set:
    ret
do_set10:
    mov   set10,1
    push  es
    mov   ah,35h
    mov   al,10h
    int   21h    ; gets the interrupt
    mov   int10ip,bx
    mov   int10cs,es
    pop   es
; now have 16 point to int16 procedure
    mov   ah,25h
    mov   al,10h
    lea   dx,int10
    int   21h
    ret
init10  endp
;
;====================================================================
;
;    INT10a Procedure - function 14 only
;
;====================================================================
;
int10a proc   near
    cmp   cs:sem10a,0
    jne   i10ago
    cmp   ah,14
    jne   i10ago
    mov   cs:sem10a,1
    jmp   i1014a
i10agoa:
    mov   cs:sem10a,0
i10ago:
    jmp   cs:[int10old]
i1014a:
    pushf
    call  cs:[int10old]
    call  put_com  ; send the byte out
    mov   cs:sem10a,0
    iret
int10a endp
;
;====================================================================
;
;    INT10 Procedure
;
;====================================================================
;
; int 10 is the emulator which squirts control codes out
; the com port so that the other programs can emulate ful screen
int10jmp dw i102,i103,i104,i105,i106,i107,i108,i109,i1010
int10  proc   near
; interrupt 10 is redirected here
    cmp   cs:sem10,0
    jne   i10go
    mov   cs:sem10,1
    cmp   ah,14
    je    i1014
    cmp   ah,2
    jl    i10goa
    cmp   ah,10
    jg    i10goa
    jmp   do_i10
i10goa:
    mov   cs:sem10,0
i10go:
    jmp   cs:[int10old]
do_i10:
    push  ax    ;12
    push  bx    ;10
    push  cx    ;8
    push  dx    ;6
    push  si    ;4
    push  bp    ;2
    mov   bp,sp   ; making ax,bp+12
    push  bx    ; gets popped
    mov   bl,ah
    mov   bh,0
    dec   bx
    dec   bx
    shl   bx,1
    mov   si,bx
    pop   bx
    call  cs:int10jmp[si]
    pop   bp
    pop   si
    pop   dx
    pop   cx
    pop   bx
    pop   ax
    mov   cs:sem10,0
    jmp   cs:[int10old]
i1014:
    pushf
    call  cs:[int10old]
    call  put_com  ; send the byte out
    mov   cs:sem10,0
    iret
i102:
; set cursor position
; send function, dx and bh
; first check to see if we are already at that cursor position
    push  dx
; get cursor position
    mov  ah,3
    pushf
    call  cs:[int10old]
    mov  ax,dx
    pop  dx
    cmp  ax,dx
    jne  go_i102
    ret
go_i102:
    mov al,0f2h  ; function 2
    call put_com
    mov al,dl
    call put_com
    mov al,dh
    call put_com
;    mov al,bh    ; no longer sending page. all pages are 0
;    call put_com
    ret
i103:
;read cursor position - null
i104:
; light pen
i105:
; active display page
i108:
; read attrib/character
    ret
i106:
; scroll up  , al,bh,cx,dx
    push ax
    mov al,0f6h  ; function 6
put_scroll:
    call put_com
    pop  ax
    call put_com
    mov al,bh
    call put_com
    mov al,cl
    call put_com
    mov al,ch
    call put_com
    mov al,dl
    call put_com
    mov al,dh
    call put_com
    ret
i107:
; scroll dn  , al,bh,cx,dx
    push  ax
    mov al,0f7h  ; function 7
    jmp  put_scroll
i109:
; put  attrib/character  ,al bx, cx
; must find out what the current character and attribute is
    cmp  cx,1
    jne  by_i109c
    push ax
    push cx
    mov  cl,al   ; byte
    mov  ch,bl   ; attrib
    mov  bh,0
    mov  ah,8   ; get char and attribute
    push  cx
    pushf
    call  cs:[int10old]
    pop   cx
; ah,al are attribute character
    cmp  ax,cx
    jne  i9note
    pop  cx
    pop  ax
    ret
i9note:
    cmp  ah,ch  ; attributes equal?
    pop  cx
    pop  ax
    je   ianote ; send just character
by_i109c:
    mov  ah,al
    mov al,0f9h  ; function 9
    cmp  cx,1
    jne  cx9
    mov al,0fbh  ; function 9 for single character
cx9:
    call put_com
    mov al,ah
    call put_com
    mov al,bl
    call put_com
;    mov al,bh    no more page numbers
;    call put_com
    cmp  cx,1    ; leave early if only one character
    jne  cx92
    ret
cx92:
    mov al,cl
    call put_com
    mov al,ch
    call put_com
    ret
i1010:
; put character only at cursor loc
; check to see if character is already there
    push ax
    push cx
    mov  cl,al
    mov  ah,8   ; get char and attribute
    mov  bx,0
    pushf
    call  cs:[int10old]
; al is character
    cmp  al,cl
    pop  cx
    pop  ax
    jne  ianote
    ret
ianote:
    mov  ah,al
    mov al,0fah  ; function 10
    cmp  cx,1
    jne  cxa
    mov  al,0fch ; function 10 for single character
cxa:
    call put_com
    mov al,ah
    call put_com
;    mov al,bh   ; no more pages
;    call put_com
    cmp  cx,1
    jne  cxab
    ret
cxab:
    mov al,cl
    call put_com
    mov al,ch
    call put_com
    ret
int10  endp
;
;====================================================================
;
;    INT0C Procedure
;    INT0B Procedure
;
;====================================================================
;
int0c  proc   near
int0b:  ; utilize same buffer. no 2 com ports at once any more
; this is the circular buffer routine called by a hardware
; interrupt to IRQ4 or 4
;
; step one, use a semaphore to limit access
    push  ax
; turn on interrupts right away?
    mov   al,1
lock xchg  cs:semaphorec,al
    cmp   al,1
    jne   ok_entry
    inc   cs:ecount
    mov  al,20h
    out  20h,al  ; this sends an End Of Interrupt command
    ; to the 8259a interrupt controller
    pop   ax
    iret
ok_entry:
    inc   cs:ecount
;  save the world
    mov  cs:isavesp,sp
    mov  cs:isavess,ss
    mov  ax,cs
    mov  ss,ax
    mov  sp,offset istack+254
    push  bx
    push  cx
    push  dx
    push  si
    push  di
    push  ds
    push  es
; we must interogate the various registers of the 8250
; to grab any information about the interrupt.
    mov   ax,cs
    mov   ds,ax
    mov   es,dx
; first check and see if it was the data register that did the
; interrupt
    call  get_com_dx
    inc   dx
    inc   dx  ; int ident reg
    in    al,dx
    sti
    test  al,4 ; was it data?
    jnz   do0cdata
    jmp   do0cstates1
do0cdata:
; we don't know the meanings of the comloc+2 register yet so ignore
    call  get_com_dx
    add  dx,5   ; line status register
    in   al,dx
    mov  comline,al
    test al,1   ; is a byte ready
    jz   do0cstates
; we have a byte ready
    call  get_com_dx
    in   al,dx  ; got the byte
; now put it to the circular buffer
; first is the buffer full?
    cli
    push  ax
    call test_flow
    pop   ax
putbuff0c:
    mov  di,buffpoint
    mov  buffer[di],al
    inc  buffcount
    inc  buffpoint
    cmp  buffpoint,buffsize
    jl   putbuff0c1
    mov  buffpoint,0
putbuff0c1:
    jmp  exit_int0c
do0cstates1:
; nothing was happening with the data so we should test the states
    call  get_com_dx
    in   al,dx
    call  get_com_dx
    add  dx,5
    in   al,dx
    mov  comline,al
do0cstates:
    call  get_com_dx
    add  dx,6
    in   al,dx
    mov  commodem,al
exit_int0c:
; reset the interrupt line
    mov  al,20h
    out  20h,al  ; this sends an End Of Interrupt command
    ; to the 8259a interrupt controller
    pop   es
    pop  ds
    pop  di
    pop  si
    pop  dx
    pop  cx
    pop  bx
    mov   ss,cs:isavess
    mov   sp,cs:isavesp
    pop  ax
    dec  cs:ecount
    mov  cs:semaphorec,0
    cmp  cs:ecount,0
    jle  int0c_leave
    jmp  int0c
int0c_leave:
    iret
get_com_dx:
    push  bx
    mov   bl,cs:comactive
    mov   bh,0
    dec   bx
    shl   bx,1
    mov   dx,cs:com1loc[bx]
    pop   bx
    ret
int0c   endp
lookup_ah proc near
; this gets the ah scan code for al for regular keyboard io stuff
   cmp  al,0
   jg   loopup
   ret
loopup:
   push  bx
   mov   bl,al
   mov   bh,0
   mov   ah,cs:scans[bx]
   pop  bx
   ret
lookup_ah endp
check_cd proc near
    push  ax
    push  cx
    push  dx
;   int 3
    mov   cx,200
ccd0:
    cli
    call  get_com_dx
    add    dx,6  ; point to modem status register
    in    al,dx ; get status
    test   al,80h ; is the cd on?
    jnz   good_cd
    loop  ccd0
bad_cd:
    sti
    stc
    pop   dx
    pop   cx
    pop   ax
    ret
good_cd:
    sti
    clc
    pop   dx
    pop   cx
    pop   ax
    ret
check_cd endp
;====================================================================
;
;    INT16 Procedure
;
;=================================================================
;
int16  proc   near
;
; this imitates the int16 except that if there is something in the
; com buffer that goes first
;
; AH = 0 is get a byte
; AH = 1 is get a byte, but don't bump the buffer count
;
    cmp  cs:sem16,0
    jne  leave16b
    mov  cs:sem16,1
    mov  cs:func16,ah
    cmp   ah,0
    je    do16_0
    cmp   ah,1
    je    do16_1
leave16:
    mov  cs:sem16,0
    mov  ah,cs:func16
    jmp   cs:[int16old]
leave16b:
    mov  ah,cs:func16
    jmp   cs:[int16old]
do16_1:  ;check for keystroke
    pushf
    mov   ah,1
    call  cs:[int16old]
    jz    no_real_key1
    jmp   leave16
no_real_key1:
    cmp   cs:comactive,1
    je    check_key1a
    cmp   cs:comactive,2
    jne   leave16
check_key1a:
    cmp  cs:buffcount,0
    jle  leave16
    jmp  save_16
do16_0:  ;wait for a keystroke
; priority is that local keyboard comes first
    mov   ah,1 ; check for a keystroke
    pushf
    call  cs:[int16old]
    jz    no_real_key
    mov   ah,0
    jmp   leave16
no_real_key:
    cmp   cs:comactive,1
    je    check_key1
    cmp   cs:comactive,2
    jne   do16_0
check_key1:
    cmp  cs:buffcount,0
    jle  do16_0
save_16:
;  save the world
    sti
    push  bx
    push  cx
    push  dx
    push  si
    push  di
    push  ds
    push  es
    mov   bx,cs
    mov   es,bx
    mov   ds,bx
;   jmp   do16get
do16get:
; this does a get from the cirular buffer
    cmp  func16,1
    je   do16get1
; 16h function 0, wait for key.
    call get_buffer
    cmp  al,0
    je   got_a_function
    call  lookup_ah
    jmp  exit_16h
got_a_function:
; al has a 0 have to wait for next byte from stream
    mov  cx,0
get_scan_code:
    sti
    cmp  buffcount,0
    jg   got_scan_code
    loop get_scan_code
got_scan_code:
    call get_buffer
    mov   ah,al
    mov   al,0
    jmp  exit_16h
do16get1:
    cli
    mov  si,bufftail
    mov  al,buffer[si]
;
    cmp   al,0
    jne   no_al_0
; have to get a scan code
    mov   cx,-1
get_scan_code2:
    sti
    cmp  buffcount,1
    jg   got_scan2
    loop get_scan_code2
got_scan2:
    inc  si
    cmp  si,buffsize
    jl   get_scan3
    mov  si,0
get_scan3:
    mov  ah,buffer[si]
    mov   al,0
    jmp  exit_16h
no_al_0:
    call  lookup_ah
exit_16h:
    pop  es
    pop  ds
    pop  di
    pop  si
    pop  dx
    pop  cx
    pop  bx
    mov  cs:sem16,0
; have to return with the zero flag set if there is no character
    cmp  cs:func16,3
; retf 2
    db 0cah,02h,00h
    iret
int16  endp
;
;====================================================================
;
;   PUT_COM writes to the com port
;
;====================================================================
;
put_com    proc near
; put_com writes bytes out the com port
    push  bx
    push  cx
    push  dx
    call  get_com_dx
; now dx has the comloc in it
    cli
    push   ax ; save the character
    add    dx,5
    mov    cx,000 ; set cx to timeout
; check first dsr and cts
    cmp    cs:flow_control,0
    je    do14try
    mov    cx,0 ; set cx to timeout a big one
check_dtr:
    inc    dx
    in    al,dx
    dec    dx
    test   al,80h  ; cd
    jz    one_more_cd
check_flow:
    and    al,30h  ; check rts and cts
    cmp    al,30h  ; check rts and cts
    je    do14try
    sti
    push   cx
    mov    cx,1000h
dtr_cloop:
    nop   ; allow interrupts and wait a bit
    loop  dtr_cloop
    pop   cx
    cli
    loop   check_dtr
    jmp    bad_put_com
one_more_cd:
    mov   cx,1000
om_cd:
    inc    dx
    in    al,dx
    dec    dx
    test   al,80h  ; cd
    jnz    check_flow
    loop   om_cd
    jmp    bad_put_com
do14try:
    in    al,dx
    test   al,20h
    jnz    put_that_byte
    loop   do14try
bad_put_com:
; failed
; line status shows transmit register not empty
; should never happen
    pop   ax
    pop   dx
    pop   cx
    pop   bx
    sti
    stc
    ret
put_that_byte:
; its ready to send a byte
    call  get_com_dx
    pop    ax
    out    dx,al
exit_14h:
    pop   dx
    pop   cx
    pop   bx
    ret
put_com    endp
;
;====================================================================
;
;   GET_BUFFER reads a character form the circular buffer
;
;====================================================================
;
get_buffer proc near
; this reads a character if there is one or sets carry for failure
    cmp  cs:buffcount,0
    jg   gb_0
    stc
    ret
gb_0:
    push  bx
    push  cx
    push  dx
    push  si
    push  di
    push  ds
    push  es
    mov   ax,cs
    mov   ds,ax
    mov   es,ax
    cli
    mov  ah,0
    mov  si,bufftail
    mov  al,buffer[si]
    inc  si
    cmp  si,buffsize
    jl   gb_1
    mov  si,0
gb_1:
    mov  bufftail,si
    dec  buffcount
    push  ax
    call  test_flow
    pop   ax
    sti
    call  lookup_ah
    clc
    pop   es
    pop   es
    pop   di
    pop   si
    pop   dx
    pop   cx
    pop   bx
    ret
test_flow:
    push  ax
    cmp  buffcount,3*(buffsize/4)
    jl   gb_0_2
    cmp  cs:bufffull,1
    je   gb_0_3
    mov  bufffull,1
; turn on flow
    call  get_com_dx
    add  dx,4   ; modem control
    mov al,09h  ; dsr and com2
    out  dx,al
    call  blip
    jmp  gb_0_3
gb_0_2:
    cmp  bufffull,0
    je   gb_0_3
;   turn off flow
    mov  bufffull,0
    call  get_com_dx
    add  dx,4   ; modem control
    mov al,0bh
    out  dx,al
gb_0_3:
    pop   ax
    ret
get_buffer endp
;
flush_com  proc near
; flushes the com buffer
    cmp  cs:buffcount,0
    je  exit_fc
    call  get_buffer
    jmp  flush_com
exit_fc:
    ret
flush_com endp
;
;====================================================================
;
;   Reset irq4 for COM1 so no interrupts
;
;====================================================================
;
unirq4  proc  near
;    sti
;    mov  al,port21h    ; old 21 register
;    out  21h,al    ; reset the interrupt masks
;   call  get_com_dx
;    inc  dx
;    inc  dx    ; set up for interrupt id register
;    in   al,dx
;    cmp  al,1    ; did it work?
;    je   dc1_3
;   call  get_com_dx
;    in   al,dx    ; flush the input register
;c1_3:
;   call  get_com_dx
;    inc  dx
;    mov  al,00h  ; no interrupts
;    out  dx,al  ; disables interrupts on this irq
; clear the interrupts
;    mov  al,20h
;    out  20h,al
; now rq4 no longer pulls at any interrupt
    mov   dx,int0cip
    mov   ax,int0ccs
    push  ds
    mov    ds,ax
    mov   ah,25h
    mov   al,0ch
    int   21h
    pop    ds
    ret
unirq4  endp
;
;====================================================================
;
;   Reset irq3 for COM2 so no interrupts
;
;====================================================================
;
unirq3  proc  near
;    sti
;    mov  al,port21h    ; old 21 register
;    out  21h,al    ; reset the interrupt masks
;    call get_com_dx
;    inc  dx
;    inc  dx    ; set up for interrupt id register
;    in   al,dx
;    cmp  al,1    ; did it work?
;    je   dc2_3
;    call get_com_dx
;    in   al,dx    ; flush the input register
;c2_3:
;    call get_com_dx
;    inc  dx
;    mov  al,00h  ; no interrupts
;    out  dx,al  ; disables interrupts on this irq
; clear the interrupts
;    mov  al,20h
;    out  20h,al
; now rq3 no longer pulls at any interrupt
    mov   dx,int0bip
    mov   ax,int0bcs
    push  ds
    mov    ds,ax
    mov   ah,25h
    mov   al,0bh
    int   21h
    pop    ds
    ret
unirq3  endp
;
;====================================================================
;
;   Set irq4 for COM1
;
;====================================================================
;
irq4  proc  near
; set irq4 to be com1
    cli
    in   al,21h   ; get the interrup mask
    mov  port21h,al  ; save to restore later
    push  ax
    call  get_com_dx  ; just wasting time to let io react
    pop   ax
    and  al,0efh  ; add ours to it
    out  21h,al
    call  setup_irq
; now rq4 pulls at int0c
    sti
    ret
irq4  endp
;
;====================================================================
;
; set up irq3 for com2
;
;====================================================================
;
irq3 proc  near
; set irq3 to be com2
    sti
    in   al,21h   ; get the interrup mask
    mov  port21h,al  ; save to restore later
    and  al,0f7h  ; add ours to it
    out  21h,al
    call  setup_irq
    ret
setup_irq:
    call get_com_dx
    add  dx,4
    mov  al,0bh   ; set dtr,rts and out2 on
    out  dx,al
    call  get_com_dx
    inc  dx
    inc  dx    ; set up for interrupt id register
    in   al,dx
    cmp  al,1    ; did it work?
    je   dc2_2
    call get_com_dx
    in   al,dx    ; flush the input register
dc2_2:
    call get_com_dx
    inc  dx
    mov  al,01h
    out  dx,al  ; enable receive data interrupts only
; clear the interrupts
    call get_com_dx ; waste a sec
    mov  al,64h
    out  20h,al
; now rq3 pulls at int0b
    ret
irq3 endp
;
;====================================================================
;
;    INIT16  Procedure
;  redirects int 16h to com routine
;=================================================================
;
init16 proc near
; now establish a new int16h
    cmp   set16,1
    jne   do_set16
    ret
do_set16:
    push  es
    mov   ah,35h
    mov   al,16h
    int   21h    ; gets the interrupt
    mov   int16ip,bx
    mov   int16cs,es
    pop   es
    mov   set16,1
; now have 16 point to int16 procedure
    mov   ah,25h
    mov   al,16h
    lea   dx,int16
    int   21h
    ret
init16 endp
;
;====================================================================
;
;    INITCOM Procedure
;
;=================================================================
;
initcom  proc  near
; first get the com1 io address
; calling sequence is same as for INT 14H
; Baud etc is determined by the caller
; COM line is in DX , 0 = 1 , 1 = 2
    push  dx
    mov   ax,40h
    push  ds
    mov   ds,ax
    mov   ax,word ptr ds:[0]
    mov   bx,word ptr ds:[2]
    pop   ds
    mov   com1loc,ax
    mov   com2loc,bx
; check to see if the irqline needs setting
    pop   bx
    push  bx
    cmp   com1on[bx],0
    je    cdone_init
    pop   bx
    ret
    jmp   cont_cominit
cdone_init:
; must set the irq line to either a or b
    cmp   bx,0
    je    do_com1
    jmp   do_com2
do_com1:
    mov  comactive,1
    mov  ah,14
    mov  al,7
    int 10h
; set interrupt to point to int0c
    push  es
    mov   ah,35h
    mov   al,0ch
    int   21h    ; gets the interrupt
    mov   int0cip,bx
    mov   int0ccs,es
    pop   es
; now have 0c point to int0c procedure
    mov   ah,25h
    mov   al,0ch
    lea   dx,int0c
    int   21h
; now turn on irq4
    jmp  cont_cominit
do_com2:
    mov  comactive,2
    mov  ah,14
    mov  al,7
    int 10h
    mov  ah,14
    mov  al,7
    int 10h
; set interrupt to point to int0c
    push  es
    mov   ah,35h
    mov   al,0bh
    int   21h    ; gets the interrupt
    mov   int0bip,bx
    mov   int0bcs,es
    pop   es
; now have 0b point to int0b procedure
    mov   ah,25h
    mov   al,0bh
    lea   dx,int0b
    int   21h
; set up irq3
; now establish that the com port is in control
cont_cominit:
    pop   bx
    push  bx
    cmp   bx,0
    je    set_com1
    jmp   set_com2
set_com1:
    call  irq4
    jmp   done_set
set_com2:
    call irq3
done_set:
    pop   bx
    mov   com1on[bx],1
; what else must be done?
    ret
initcom  endp
ATDT    db 'ATDT'
    db  16 dup(0)
telnum  db '3532157'
    db  16 dup(0)
DIAL  PROC NEAR
; this procedure dials the number in telnum using the ATDT command
;raise modem
    call get_com_dx
    add  dx,4
    mov  al,3  ; bring em high
    out  dx,al
;
    mov  cx,1
    call waitsec   ; waits for number of seconds in cx
    add  dx,4
    mov  al,2
    out  dx,al
    mov  bx,0
    mov   cx,2
    call  waitsec
;raise modem
    call get_com_dx
    add  dx,4
    mov  al,3
    out  dx,al
    mov   cx,2
    call  waitsec
; send ATDT
    lea  si,atdt
    call putcom
    lea  si,telnum
    call putcom
    mov  al,13
    call putbyte
;
; DELAY 5 SEC
    mov cx,2
    call  waitsec
    ret
waitsec:
; uses bios calls to wait a second
    push  cx
; get the current time in ticks
    mov   ah,00h
    int   1ah
; the registers are set as follows:
   ;cx is high order clock count
   ;dx is low order clock count
    add   dx,18
    and   dx,7fffh
    mov   bx,dx
wloop1:
    push  bx
    mov   ah,00h
    int   1ah
    pop   bx
    and   dx,7fffh
    cmp   dx,bx
    jl    wloop1
    pop   cx
    loop  waitsec
    ret
dial    endp
transmit proc near
; this will send a string same as putcom but will use cx
    call  check_cd
    jnc   tr_loop
    stc
    ret
tr_loop:
    push  cx
    lodsb
    call putbyte
    pop   cx
    jc    tr_end
    loop  tr_loop
    call  check_cd
tr_end:
    ret
transmit endp
putcom  proc  near
; puts a string from si out the port until a 0 is found
putloop:
    lodsb
    cmp  al,0
    je   putret
    call putbyte
    jc   putret
    jmp  putloop
putret:
    ret
putbyte:
; this puts the byte in al out the com port
; uses put_com
    call  put_com
    ret
putcom  endp

blip    proc  near
    push  ax
    push  bx
    push  cx
    push  dx
    push  si
    mov   bx,400  ; starting freq
    mov   cx,800  ; ending frequency
; turn on tone
i0:
    push  cx
    push  bx
    in    al,61h
    or    al,3
    out    61h,al
; loop parameters
    mov    si,50
    cmp    bx,cx
    jle    i1
    neg    si
i1:
    mov    cx,bx
 ; set frequency
    mov    dx,12h
    mov    ax,34deh
    div    cx
    mov    cx,ax
; set tone
    mov    al,cl
    out    42h,al
    mov    al,ch
    out    42h,al
i2:
    nop
    nop
    loop i2
    pop    bx
    pop    cx
    cmp    bx,cx
    jg    ip3
    add    bx,si
    jmp    i0
ip3:
 ;turn off tone
    in  al,61h
    and al,11111100b
    out 61h,al
    pop   si
    pop   dx
    pop   cx
    pop   bx
    pop   ax
    ret
blip    endp



