;
;====================================================================
; this area is for int 14h and int 0ch                              |
; It consists of 5 routines:                                        |
;                                                                   |
;   PROCEDURE INITCOM initializes the serial port vectors for       |
;   the serial port requested. The calling format is the            |
;   same as the ah 0 call of int 14                                 |
;                                                                   |
;   PROCEDURE INT0B is a 2k circular buffer which captures com ints |
;                                                                   |
;   PROCEDURE INT0C same as int0b except for com1                   |
;                                                                   |
;   PROCEDURE INT14 is a replacement for the BIOS routine           |
;                                                                   |
;   PROCEDURE DECOM deinitializes the serial port vectors           |
;                                                                   |
; public data consists of the names of the routines                 |
    PUBLIC INITCOM,DECOM
;====================================================================
;
CSEG   SEGMENT PARA PUBLIC 'CODE'
   ASSUME CS:CSEG,DS:CSEG,SS:CSEG,ES:CSEG ;ALREADY SET BY DOS LOADER
;====================================================================
;                  DATA AREAS                                       |
;====================================================================
com1loc  dw  0
com2loc  dw  0
com1on   db  0
com2on   db  0
set14    db  0
int14old equ this dword
int14ip  dw  0
int14cs  dw  0
int0cip  dw  0
int0ccs  dw  0
int0bip  dw  0
int0bcs  dw  0
buffsize equ  1024
buffer1  db   buffsize dup(0)
buff1count dw  0
buff1point dw  0
buff1tail  dw  0
buff1full  db  0
com1line   db  0
com1modem  db  0
buffer2  db   buffsize dup(0)
buff2count dw  0
buff2point dw  0
buff2tail  dw  0
buff2full  db  0
com2line   db  0
com2modem  db  0
semaphore1 db  0
;
;====================================================================
;                  PROCEDURES                                       |
;====================================================================
;
;    INITCOM Procedure
;
;=================================================================
;
initcom  proc  near
; first get the com1 io address
     push  ax
     push  cx
     push  dx
     push  ax
     mov   ax,40h
     push  ds
     mov   ds,ax
     mov   ax,word ptr ds:[0]
     mov   bx,word ptr ds:[2]
     pop   ds
     mov   com1loc,ax
     mov   com2loc,bx
; now initialize the com port
     pop   ax
     mov   ah,0
     push  dx
     int   14h
; now establish a new int14h
; but must check if we already had this set
     cmp   set14,1
     je    noset14
     push  es
     mov   ah,35h
     mov   al,14h
     int   21h     ; gets the interrupt
     mov   int14ip,bx
     mov   int14cs,es
     pop   es
     mov   set14,1
; now have 14 point to int14 procedure
     mov   ah,25h
     mov   al,14h
     lea   dx,int14
     int   21h
noset14:
; check to see if the irqline needs setting
     pop   bx
     push  bx
     cmp   com1on[bx],1
     jne   cdone_init
     jmp   done_init
cdone_init:
; must set the irq line to either a or b
     cmp   bx,0
     je    do_com1
     jmp   do_com2
do_com1:
; set interrupt to point to int0c
     push  es
     mov   ah,35h
     mov   al,0ch
     int   21h     ; gets the interrupt
     mov   int0cip,bx
     mov   int0ccs,es
     pop   es
; now have 0c point to int0c procedure
     mov   ah,25h
     mov   al,0ch
     lea   dx,int0c
     int   21h
; set irq4 to be com1
     sti
     in   al,21h   ; get the interrup mask
     and  al,0efh  ; add ours to it
     out  21h,al
     mov  dx,com1loc
     add  dx,4
     mov  al,0bh   ; set dtr,rts and out2 on
     out  dx,al
     dec  dx
     dec  dx       ; set up for interrupt id register
     nop
     nop
     in   al,dx
     cmp  al,1     ; did it work?
     je   dc1_2
     mov  dx,com1loc
     in   al,dx    ; flush the input register
dc1_2:
     mov  dx,com1loc
     inc  dx
     mov  al,01h  ; 1 means data interrupts, F means all?
     out  dx,al  ; enable receive data interrupts only
; clear the interrupts
     mov  al,64h
     out  20h,al
; now rq4 pulls at int0c
     jmp  cont_cominit
do_com2:
; set interrupt to point to int0c
     push  es
     mov   ah,35h
     mov   al,0bh
     int   21h     ; gets the interrupt
     mov   int0bip,bx
     mov   int0bcs,es
     pop   es
; now have 0b point to int0b procedure
     mov   ah,25h
     mov   al,0bh
     lea   dx,int0b
     int   21h
; set irq3 to be com2
     sti
     in   al,21h   ; get the interrup mask
     and  al,0f7h  ; add ours to it
     out  21h,al
     mov  dx,com2loc
     add  dx,4
     mov  al,0bh   ; set dtr,rts and out2 on
     out  dx,al
     dec  dx
     dec  dx       ; set up for interrupt id register
     in   al,dx
     cmp  al,1     ; did it work?
     je   dc2_2
     mov  dx,com2loc
     in   al,dx    ; flush the input register
dc2_2:
     mov  dx,com2loc
     inc  dx
     mov  al,0fh
     out  dx,al  ; enable receive data interrupts only
; clear the interrupts
     mov  al,64h
     out  20h,al
; now rq3 pulls at int0b
; now establish that the com port is in control
cont_cominit:
     pop   bx
     mov   com1on[bx],1
; what else must be done?
done_init:
     pop   dx
     pop   cx
     pop   ax
     ret
initcom  endp
;=================================================================
;
;    INT0C Procedure
;
;=================================================================
;
int0c  proc   near
; this is the circular buffer routine called by a hardware
; interrupt to IRQ4
;
;  save the world
    push  ax
    push  bx
    push  cx
    push  dx
    push  si
    push  di
    push  ds
    push  es
; we must interogate the various registers of the 8250
; to grab any information about the interrupt.
    mov   ax,cs
    mov   ds,ax
    mov   es,dx
;   mov   dx,com1loc
;   add   dx,4
;   mov   al,0bh  ; restore the com lines
;   out   dx,al
; first check and see if it was the data register that did the
; interrupt
    mov   dx,com1loc
    inc   dx
    inc   dx  ; int ident reg
    in    al,dx
    sti
    test  al,4 ; was it data?
    jnz   do0cdata
    jmp   do0cstates1
do0cdata:
; we don't know the meanings of the comloc+2 register yet so ignore
    mov  dx,com1loc
    add  dx,5   ; line status register
    in   al,dx
    mov  com1line,al
    test al,1   ; is a byte ready
    jz   do0cstates
; we have a byte ready
    mov  dx,com1loc
    nop
    in   al,dx  ; got the byte
; now put it to the circular buffer
; first is the buffer full?
    cli
    cmp  buff1count,buffsize
    jl   putbuff0c
    mov  buff1full,1
    jmp  do0cstates1
putbuff0c:
    mov  di,buff1point
    mov  buffer1[di],al
    inc  buff1point
    inc  buff1count
    cmp  buff1point,buffsize
    jl   putbuff0c1
    mov  buff1point,0
putbuff0c1:
    sti
    jmp  exit_int0c
do0cstates1:
; nothing was happening with the data so we should test the states
    mov  dx,com1loc
    in   al,dx
    nop
    nop
    add  dx,5
    in   al,dx
    mov  com1line,al
do0cstates:
    mov  dx,com1loc
    add  dx,6
    nop
    in   al,dx
    mov  com1modem,al
exit_int0c:
; reset the interrupt line
    mov  al,20h
    out  20h,al  ; this sends an End Of Interrupt command
                 ; to the 8259a interrupt controller
    sti
    pop   es
    pop  ds
    pop  di
    pop  si
    pop  dx
    pop  cx
    pop  bx
    pop  ax
    iret
int0c   endp
;====================================================================
;
;    INT0B Procedure
;
;=================================================================
;
int0b  proc   near
; this is the circular buffer routine called by a hardware
; interrupt to IRQ3
;
;  save the world
    push  ax
    push  bx
    push  cx
    push  dx
    push  si
    push  di
    push  ds
    push  es
; we must interogate the various registers of the 8250
; to grab any information about the interrupt.
    mov   ax,cs
    mov   ds,ax
    mov   es,dx
;   mov   dx,com2loc
;   add   dx,4
;   mov   al,0bh  ; restore the com lines
;   out   dx,al
; first check and see if it was the data register that did the
; interrupt
    mov   dx,com2loc
    inc   dx
    inc   dx  ; int ident reg
    in    al,dx
    sti
    test  al,4 ; was it data?
    jnz   do0bdata
    jmp   do0bstates1
do0bdata:
; we don't know the meanings of the comloc+2 register yet so ignore
    mov  dx,com2loc
    add  dx,5   ; line status register
    in   al,dx
    mov  com2line,al
    test al,1   ; is a byte ready
    jz   do0bstates
; we have a byte ready
    mov  dx,com2loc
    nop
    in   al,dx  ; got the byte
; now put it to the circular buffer
; first is the buffer full?
    cli
    cmp  buff2count,buffsize
    jl   putbuff0b
    mov  buff2full,1
    jmp  do0bstates1
putbuff0b:
    mov  di,buff2point
    mov  buffer2[di],al
    inc  buff2point
    inc  buff2count
    cmp  buff2point,buffsize
    jl   putbuff0b1
    mov  buff2point,0
putbuff0b1:
    sti
    jmp  exit_int0b
do0bstates1:
; nothing was happening with the data so we should test the states
    mov  dx,com2loc
    in   al,dx
    nop
    nop
    add  dx,5
    in   al,dx
    mov  com2line,al
do0bstates:
    mov  dx,com2loc
    add  dx,6
    nop
    in   al,dx
    mov  com2modem,al
exit_int0b:
; reset the interrupt line
    mov  al,20h
    out  20h,al  ; this sends an End Of Interrupt command
                 ; to the 8259a interrupt controller
    sti
    pop   es
    pop  ds
    pop  di
    pop  si
    pop  dx
    pop  cx
    pop  bx
    pop  ax
    iret
int0b   endp
;====================================================================
;
;    INT14 Procedure
;
;=================================================================
;
int14  proc   near
;
; this imitates the int14 with a few exceptions
    cmp   ah,0
    jne   noah0
leave14:
    jmp   cs:int14old
noah0:
    cmp   ah,3
    jg    leave14
    cmp   dx,1
    je   dx1
    cmp   dx,0
    jne  leave14
    cmp   cs:com1on,1
    jne  leave14
    jmp  do14
dx1:
    cmp   cs:com2on,1
    jne  leave14
do14:
;  save the world
    sti
    push  bx
    push  cx
    push  dx
    push  si
    push  di
    push  ds
    push  es
    mov   bx,cs
    mov   es,bx
    mov   ds,bx
    cmp   ah,3
    je    do14status
    cmp   ah,1
    je    do14send
    cmp   ah,2
    je    do14get
exit_14h:
    pop   es
    pop  ds
    pop  di
    pop  si
    pop  dx
    pop  cx
    pop  bx
    iret
do14status:
    cmp  dx,1
    jne  do14stat1
    mov  ah,com2line
    mov  al,com2modem
    jmp  exit_14h
do14stat1:
    mov  ah,com1line
    mov  al,com1modem
    jmp  exit_14h
do14send:
; this is the only one where we realy have to work
     cmp  dx,0
     je   do14send1
     mov  dx,com2loc
     jmp  do14send2
do14send1:
     mov  dx,com1loc
do14send2:
; now dx has the comloc in it
     cli
     push   ax ; save the character
     add    dx,4
;    mov    al,0bh
;    out    dx,al   ; sets high dtr out2 and rts
     inc    dx
     mov    cx,2000 ; set cx to timeout
do14try:
     in     al,dx
     test   al,20h
     jnz    put_that_byte
     loop   do14try
; lost this one
     sub    dx,5
     cmp    dx,com1loc
     jne    do14fail1
     mov    com2line,al
     jmp    do14fail2
do14fail1:
     mov    com2line,al
do14fail2:
     mov    ah,al
     or     ah,80h
     pop    bx
     sti
     jmp    exit_14h
put_that_byte:
; its ready to send a byte
     mov    bx,ax
     sub    dx,5
     cmp    dx,com1loc
     jne    do14succ1
     mov    com2line,al
     jmp    do14succ2
do14succ1:
     mov    com2line,al
do14succ2:
     pop    ax
     out    dx,al
     mov    ah,bh
     and    ah,7fh
     jmp    exit_14h
do14get:
; this does a get from the cirular buffer
    cmp  dx,1
    jne  do14get1
    jmp  do14get2
do14get1:
    cli
    cmp  buff1count,0
    jg   got14char1
    mov  ah,com1line
    and  ah,9eh
    or   ah,80h
    sti
    jmp  exit_14h
got14char1:
    mov  ah,0
    mov  si,buff1tail
    mov  al,buffer1[si]
    inc  si
    cmp  si,buffsize
    jl   g14c12
    mov  si,0
g14c12:
    mov  buff1tail,si
    dec  buff1count
    sti
    jmp  exit_14h
do14get2:
    cli
    cmp  buff2count,0
    jg   got14char2
    mov  ah,com2line
    and  ah,9eh
    or   ah,80h
    sti
    jmp  exit_14h
got14char2:
    mov  ah,0
    mov  si,buff2tail
    mov  al,buffer2[si]
    inc  si
    cmp  si,buffsize
    jl   g14c22
    mov  si,0
g14c22:
    mov  buff2tail,si
    dec  buff2count
    sti
    jmp  exit_14h
int14  endp
;====================================================================
;
;    DECOM Procedure
;
;=================================================================
;
decom  proc  near
; this routine turns off the interrupts and resets all to original
; values
; restore int 14
     mov   dx,int14ip
     mov   ax,int14cs
     push  ds
     mov   ds,ax
     mov   ah,25h
     mov   al,14h
     int   21h
     pop   ds
    cmp com1on,1
    jne com1off
    in  al,21h
    or  al,10h
    out 21h,al
; now drop dtr and and out2
    mov  dx,com1loc
   inc   dx
    mov  al,2
   out   dx,al
; now restore the com1 interrupt
     mov   dx,int0cip
     mov   ax,int0ccs
     push  ds
     mov   ds,ax
     mov   ah,25h
     mov   al,0ch
     int   21h
     pop   ds
com1off:
    cmp com2on,1
    jne com2off
    in  al,21h
    or  al,08h
    out 21h,al
; now restore the com2 interrupt
     mov   dx,int0bip
     mov   ax,int0bcs
     push  ds
     mov   ds,ax
     mov   ah,25h
     mov   al,0bh
     int   21h
     pop   ds
; now drop dtr and and out2
    mov  dx,com2loc
   inc   dx
    mov  al,2
   out   dx,al
com2off:
    mov  word ptr com1on,0
    mov  set14,0
    ret
decom  endp
CSEG   ENDS
   END
